﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Runtime.CompilerServices;
using System.Text;
using System.Text.Json;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace TunnelDBJsonViewer
{
    public partial class FormAPISample : Form
    {
        //最新簡易参照
        const string BASE_URL_LASTEST = "https://road-structures-db-tunnel.mlit.go.jp/xROAD/api/v1/tunnels/lastest";
        //簡易参照
        const string BASE_URL_TUNNELS = "https://road-structures-db-tunnel.mlit.go.jp/xROAD/api/v1/tunnels";
        //高度参照
        const string BASE_URL_SEARCH = "https://road-structures-db-tunnel.mlit.go.jp/xROAD/api/v1/tunnels/search";

        public FormAPISample()
        {
            InitializeComponent();

            cbxAPIKind.SelectedIndex = 0; //最新簡易参照

            //TSL1.2を有効にする
            ServicePointManager.SecurityProtocol = SecurityProtocolType.Tls | SecurityProtocolType.Tls11 | SecurityProtocolType.Tls12;
        }

        /// <summary>
        /// 参照実行
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private async void btnRequest_Click(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(tbxAPIKey.Text))
            {
                MessageBox.Show("APIKeyを入力してください。");
                return;
            }

            switch (cbxAPIKind.SelectedIndex)
            {
                case 0:
                    //最新データ参照
                    await SearchSimple(BASE_URL_LASTEST);
                    break;
                case 1:
                    //最新データ参照
                    await SearchSimple(BASE_URL_TUNNELS);
                    break;
                case 2:
                    //高度参照
                    await SearchDetail(BASE_URL_SEARCH);
                    break;
                default:
                    MessageBox.Show("APIの種類を選択してください。");
                    break;
            }

        }

        /// <summary>
        /// 高度参照
        /// </summary>
        /// <returns></returns>
        private async Task SearchDetail(string url)
        {

            //クエリー
            var querys = new List<SearchQuery>();
            //施設名で絞り込み
            if (!string.IsNullOrEmpty(tbxName.Text))
            {
                querys.Add(new SearchQuery() 
                    {
                        key = "syogen.shisetsu.meisyou",
                        value = tbxName.Text,
                        op = "7" //含む
                });
            }
            //年度で絞り込み
            if (nudNendo.Value != 0)
            {
                querys.Add(new SearchQuery()
                {
                    key = "tenken.nendo",
                    value = nudNendo.Value.ToString(),
                    op = "1" //等しい
                });
            }
           
            //パラメータ全体
            var parameters = new
            {
                querys = querys.ToArray(),
                offset = 0,
                limit = nudLimit.Value.ToString(),
                dataKind = new
                {
                    syogen_detail = 1, //緒元詳細出力
                    tenken_detail = 1  //点検詳細出力
                    //・・・・
                }
            };

           
            var jsonContent = new StringContent(
                JsonSerializer.Serialize(parameters),
                Encoding.UTF8,
                "application/json");


            //リクエスト
            var request = new HttpRequestMessage(HttpMethod.Post, url);
            request.Headers.Add("API-key", tbxAPIKey.Text);//APIキーを設定する
            request.Content = jsonContent;
            

            tbxResult.Text = "検索中・・・";
            using (var client = new HttpClient())
            {
                var response = await client.SendAsync(request);
                var result = await response.Content.ReadAsStringAsync();
                tbxResult.Text = ToReadable(result);
            }
        }

        /// <summary>
        /// 簡易参照
        /// </summary>
        /// <returns></returns>
        private async Task SearchSimple(string url)
        {

            //パラメータ
            var parameters = new Dictionary<string, string>()
            {
                { "limit", nudLimit.Value.ToString()}
            };

            //施設名で絞り込み
            if (!string.IsNullOrEmpty(tbxName.Text))
            {
                parameters.Add("name", tbxName.Text);
            }
            //年度で絞り込み
            if (nudNendo.Value != 0)
            {
                parameters.Add("nendo", nudNendo.Value.ToString());
            }

            //パラメータ付きのURL
            var sUrl = url + $"?{await new FormUrlEncodedContent(parameters).ReadAsStringAsync()}";
            var request = new HttpRequestMessage(HttpMethod.Get, sUrl);
            request.Headers.Add("API-key", tbxAPIKey.Text);//APIキーを設定する

            tbxResult.Text = "検索中・・・";
            using (var client = new HttpClient())
            {
                var response = await client.SendAsync(request);
                var result = await response.Content.ReadAsStringAsync();
                tbxResult.Text = ToReadable(result);
            }
        }

        /// <summary>
        /// Jsonを整形
        ///　参考 https://baba-s.hatenablog.com/entry/2018/05/06/120400
        /// </summary>
        /// <param name="json"></param>
        /// <returns></returns>
        public static string ToReadable(string json)
        {
            if (string.IsNullOrEmpty(json)) return json;

            int i = 0;
            int indent = 0;
            int quoteCount = 0;
            int position = -1;
            var sb = new StringBuilder();
            int lastindex = 0;

            while (true)
            {
                if (i > 0 && json[i] == '"' && json[i - 1] != '\\') quoteCount++;

                if (quoteCount % 2 == 0)
                {
                    if (json[i] == '{' || json[i] == '[')
                    {
                        indent++;
                        position = 1;
                    }
                    else if (json[i] == '}' || json[i] == ']')
                    {
                        indent--;
                        position = 0;
                    }
                    else if (json.Length > i && json[i] == ',' && json[i + 1] == '"')
                    {
                        position = 1;
                    }
                    if (position >= 0)
                    {
                        sb.AppendLine(json.Substring(lastindex, i + position - lastindex));
                        sb.Append(new string(' ', indent * 4));
                        lastindex = i + position;
                        position = -1;
                    }
                }

                i++;

                if (json.Length <= i)
                {
                    sb.Append(json.Substring(lastindex));
                    break;
                }

            }
            return sb.ToString();
        }

        public class SearchQuery
        {
            public string key { get; set; }
            public string value { get; set; }
            public string op { get; set; }
        }
    }
}
